/*
 * Copyright (c) Xerox Corporation 1997. All rights reserved.
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or
 * derivative work. Xerox grants no other licenses expressed or
 * implied. The Xerox trade name should not be used in any advertising
 * without its written permission. 
 *
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *
 * These notices must be retained in any copies of any part of this
 * software. 
 */

#include "agent.h"
#include "Tcl.h"
#include "packet.h"
#include "ip.h"
#include "rtp.h"
#include "adaptive-receiver.h"
#include <math.h>

#define myabs(r) (r<0)?-r:r

AdaptiveRcvr::AdaptiveRcvr() : Agent(-1),npkts_(0),ndelay_(0),lasttimestamp_(0),nvar_(0.0),lastdelay_(-1)
{
  bind("npkts_",&npkts_);
  bind("ndelay_",&ndelay_);
  bind("nvar_",&nvar_);
  bind("off_rtp_",&off_rtp_);
}



int AdaptiveRcvr::command(int argc, const char*const* argv)
{
  if (argc==2) {
    if (strcmp(argv[1],"get_avgdel") == 0) {
      ndelay_= ndelay_*1.0/npkts_;
      return TCL_OK;
    }
    if (strcmp(argv[1],"get_avgdis") == 0) {
      nvar_= (nvar_*1.0/npkts_);
      return TCL_OK;
    }
  }
  return (Agent::command(argc,argv));
}

void AdaptiveRcvr::recv(Packet *pkt,Handler*)
{
  int delay;
  int seq_no;
  hdr_cmn* ch= (hdr_cmn*)pkt->access(off_cmn_);
  hdr_ip* iph = (hdr_ip*)pkt->access(off_ip_);
  hdr_rtp *rh=(hdr_rtp*) pkt->access(off_rtp_);

  seq_no= rh->seqno();
  register u_int32_t send_time = (int)(ch->timestamp());

  u_int32_t local_time= (u_int32_t)(Scheduler::instance().clock() * SAMPLERATE);
  delay=adapt(pkt,local_time);
  compute_stats(delay,send_time,local_time);
  //if ((local_time/8000.0 >= 2000 ) && (local_time/8000.0) <=2500)
  //print_stats(iph->flowid(),seq_no,send_time,local_time,local_time+delay);
  Packet::free(pkt);
}

void AdaptiveRcvr::print_stats(int fid,int seq_no,int send_time,int rcv_time,int play_time)
{
    fprintf (stderr,"%d %d %f %f %f %f\n",fid,seq_no,(rcv_time)/8000.0,(send_time)/8000.0,(rcv_time-send_time)/8.0,play_time/8.0);
    fflush(stderr);
}


void AdaptiveRcvr::compute_stats(double delay,u_int32_t curr_timestamp,u_int32_t rcv_time)
{
  double tmp;
  double tdelay = delay+rcv_time-curr_timestamp;
  if (lastdelay_ == -1)
    lastdelay_=tdelay;

  if (curr_timestamp-lasttimestamp_ != 0) {
    npkts_++;
    ndelay_ += tdelay;
    //tmp=nvar_;
    tmp=myabs((tdelay-lastdelay_)/(curr_timestamp-lasttimestamp_));
    if (tmp > 2) tmp = 2; 
    if (lastdelay_ == -1000) // indicates packet loss
      tmp = 6;
    //if (tmp !=0)
    //  fprintf(stderr,"nvar %f \n",tmp);
    nvar_ += tmp;
    //if (nvar_-tmp != 0)
    //  fprintf(stderr,"nvar %f \n",nvar_-tmp);
    
  }
  lastdelay_=tdelay;
  lasttimestamp_=curr_timestamp;
  
}
